@@ -46,6 +46,10 @@ typedef int Bool;
 
 #define UNUSED_ARG(arg)  (void)arg
 
+#if defined(__MINGW32__) || defined(_MSC_VER)
+    #define PYUV_WINDOWS
+#endif
+
 #define ASSERT(x)                                                           \
     do {                                                                    \
         if (!(x)) {                                                         \
@@ -55,11 +59,31 @@ typedef int Bool;
         }                                                                   \
     } while(0)                                                              \
 
-#define UV_LOOP(x) x->loop->uv_loop
+#define UV_LOOP(x) (x)->loop->uv_loop
 
-#if defined(__MINGW32__) || defined(_MSC_VER)
-    #define PYUV_WINDOWS
-#endif
+#define UV_HANDLE(x) ((Handle *)x)->uv_handle
+
+#define UV_HANDLE_CLOSED(x) (!UV_HANDLE(x) || uv_is_closing(UV_HANDLE(x)))
+
+#define UV_HANDLE_LOOP(x) UV_LOOP((Handle *)x)
+
+#define RAISE_IF_HANDLE_CLOSED(obj, exc_type, retval)                       \
+    do {                                                                    \
+        if (UV_HANDLE_CLOSED(obj)) {                                        \
+            PyErr_SetString(exc_type, "");                                  \
+            return retval;                                                  \
+        }                                                                   \
+    } while(0)                                                              \
+
+#define RAISE_UV_EXCEPTION(loop, exc_type)                                          \
+    do {                                                                            \
+        uv_err_t err = uv_last_error(loop);                                         \
+        PyObject *exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));     \
+        if (exc_data != NULL) {                                                     \
+            PyErr_SetObject(exc_type, exc_data);                                    \
+            Py_DECREF(exc_data);                                                    \
+        }                                                                           \
+    } while(0)                                                                      \
 
 
 /* Python types definitions */
@@ -67,99 +91,84 @@ typedef int Bool;
 /* Loop */
 typedef struct {
     PyObject_HEAD
-    PyObject *data;
+    PyObject *weakreflist;
+    PyObject *dict;
     uv_loop_t *uv_loop;
     int is_default;
 } Loop;
 
 static PyTypeObject LoopType;
 
-/* Async */
+/* Handle */
 typedef struct {
     PyObject_HEAD
+    PyObject *weakreflist;
+    PyObject *dict;
     Loop *loop;
     PyObject *on_close_cb;
+    uv_handle_t *uv_handle;
+} Handle;
+
+static PyTypeObject HandleType;
+
+/* Async */
+typedef struct {
+    Handle handle;
     PyObject *callback;
-    PyObject *data;
-    uv_async_t *uv_handle;
 } Async;
 
 static PyTypeObject AsyncType;
 
 /* Timer */
 typedef struct {
-    PyObject_HEAD
-    Loop *loop;
-    PyObject *on_close_cb;
+    Handle handle;
     PyObject *callback;
-    PyObject *data;
-    uv_timer_t *uv_handle;
 } Timer;
 
 static PyTypeObject TimerType;
 
 /* Prepare */
 typedef struct {
-    PyObject_HEAD
-    Loop *loop;
-    PyObject *on_close_cb;
+    Handle handle;
     PyObject *callback;
-    PyObject *data;
-    uv_prepare_t *uv_handle;
 } Prepare;
 
 static PyTypeObject PrepareType;
 
 /* Idle */
 typedef struct {
-    PyObject_HEAD
-    Loop *loop;
-    PyObject *on_close_cb;
+    Handle handle;
     PyObject *callback;
-    PyObject *data;
-    uv_idle_t *uv_handle;
 } Idle;
 
 static PyTypeObject IdleType;
 
 /* Check */
 typedef struct {
-    PyObject_HEAD
-    Loop *loop;
-    PyObject *on_close_cb;
+    Handle handle;
     PyObject *callback;
-    PyObject *data;
-    uv_check_t *uv_handle;
 } Check;
 
 static PyTypeObject CheckType;
 
 /* Signal */
 typedef struct {
-    PyObject_HEAD
-    Loop *loop;
-    PyObject *on_close_cb;
-    PyObject *data;
-    uv_prepare_t *uv_handle;
+    Handle handle;
 } Signal;
 
 static PyTypeObject SignalType;
 
-/* IOStream */
+/* Stream */
 typedef struct {
-    PyObject_HEAD
-    Loop *loop;
+    Handle handle;
     PyObject *on_read_cb;
-    PyObject *on_close_cb;
-    PyObject *data;
-    uv_stream_t *uv_handle;
-} IOStream;
+} Stream;
 
-static PyTypeObject IOStreamType;
+static PyTypeObject StreamType;
 
 /* TCP */
 typedef struct {
-    IOStream iostream;
+    Stream stream;
     PyObject *on_new_connection_cb;
 } TCP;
 
@@ -167,7 +176,7 @@ static PyTypeObject TCPType;
 
 /* Pipe */
 typedef struct {
-    IOStream iostream;
+    Stream stream;
     PyObject *on_new_connection_cb;
 } Pipe;
 
@@ -175,28 +184,62 @@ static PyTypeObject PipeType;
 
 /* TTY */
 typedef struct {
-    IOStream iostream;
+    Stream stream;
 } TTY;
 
 static PyTypeObject TTYType;
 
 /* UDP */
 typedef struct {
-    PyObject_HEAD
-    Loop *loop;
+    Handle handle;
     PyObject *on_read_cb;
-    PyObject *on_close_cb;
-    PyObject *data;
-    uv_udp_t *uv_handle;
 } UDP;
 
 static PyTypeObject UDPType;
 
+/* Poll */
+typedef struct {
+    Handle handle;
+    PyObject *callback;
+} Poll;
+
+static PyTypeObject PollType;
+
+/* Process */
+typedef struct {
+    PyObject_HEAD
+    PyObject *stream;
+    int fd;
+    int flags;
+} StdIO;
+
+static PyTypeObject StdIOType;
+
+typedef struct {
+    Handle handle;
+    PyObject *on_exit_cb;
+    PyObject *stdio;
+} Process;
+
+static PyTypeObject ProcessType;
+
+/* FSEvent */
+typedef struct {
+    PyObject_HEAD
+    PyObject *weakreflist;
+    Loop *loop;
+    PyObject *on_fsevent_cb;
+    PyObject *on_close_cb;
+    PyObject *data;
+    uv_fs_event_t *uv_handle;
+} FSEvent;
+
+static PyTypeObject FSEventType;
+
 /* DNSResolver */
 typedef struct {
     PyObject_HEAD
     Loop *loop;
-    PyObject *data;
     ares_channel channel;
 } DNSResolver;
 
@@ -210,32 +253,71 @@ typedef struct {
 
 static PyTypeObject ThreadPoolType;
 
-/* Process */
+
+/* Some helper stuff */
+
+/* Temporary hack: libuv should provide uv_inet_pton and uv_inet_ntop. */
+#ifdef PYUV_WINDOWS
+    #include <inet_net_pton.h>
+    #include <inet_ntop.h>
+    #define uv_inet_pton ares_inet_pton
+    #define uv_inet_ntop ares_inet_ntop
+#else /* __POSIX__ */
+    #include <arpa/inet.h>
+    #define uv_inet_pton inet_pton
+    #define uv_inet_ntop inet_ntop
+#endif
+
+
+/* Add a type to a module */
+static int
+PyUVModule_AddType(PyObject *module, const char *name, PyTypeObject *type)
+{
+    if (PyType_Ready(type)) {
+        return -1;
+    }
+    Py_INCREF(type);
+    if (PyModule_AddObject(module, name, (PyObject *)type)) {
+        Py_DECREF(type);
+        return -1;
+    }
+    return 0;
+}
+
+
+/* Add a type to a module */
+static int
+PyUVModule_AddObject(PyObject *module, const char *name, PyObject *value)
+{
+    Py_INCREF(value);
+    if (PyModule_AddObject(module, name, value)) {
+        Py_DECREF(value);
+        return -1;
+    }
+    return 0;
+}
+
+
+/* borrowed from pyev */
+#ifdef PYUV_WINDOWS
+/* avoid including socketmodule.h (not available anyway) */
 typedef struct {
-    PyObject_HEAD
-    Loop *loop;
-    PyObject *on_close_cb;
-    PyObject *on_exit_cb;
-    PyObject *stdin_pipe;
-    PyObject *stdout_pipe;
-    PyObject *stderr_pipe;
-    PyObject *data;
-    uv_process_t *uv_handle;
-} Process;
+    PyTypeObject *Sock_Type;
+    PyObject *error;
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 2
+    PyObject *timeout_error;
+#endif
+} PySocketModule_APIObject;
 
-static PyTypeObject ProcessType;
+static PySocketModule_APIObject PySocketModule;
 
-/* FSEvent */
-typedef struct {
-    PyObject_HEAD
-    Loop *loop;
-    PyObject *on_fsevent_cb;
-    PyObject *on_close_cb;
-    PyObject *data;
-    uv_fs_event_t *uv_handle;
-} FSEvent;
+#define PYUV_MAXSTDIO 2048
 
-static PyTypeObject FSEventType;
+/* Convert a Windows handle into a usable file descriptor */
+#define PYUV_WIN32_HANDLE_TO_FD(handle) _open_osfhandle (handle, 0)
+#define PYUV_FD_TO_WIN32_HANDLE(fd) _get_osfhandle (fd)
+
+#endif
 
 #endif
 
