@@ -64,15 +64,19 @@ class ProcessTest(unittest2.TestCase):
             self.exit_cb_called += 1
             proc.close(handle_close_cb)
         def stdout_read_cb(handle, data, error):
-            self.received_output = data.strip()
+            if data is not None:
+                self.received_output = data.strip()
             handle.close(handle_close_cb)
         loop = pyuv.Loop.default_loop()
         stdout_pipe = pyuv.Pipe(loop)
+        stdio = []
+        stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
+        stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
         proc = pyuv.Process(loop)
         if sys.platform == 'win32':
-            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_stdout.py"], exit_callback=proc_exit_cb, stdout=stdout_pipe)
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_stdout.py"], exit_callback=proc_exit_cb, stdio=stdio)
         else:
-            proc.spawn(file="./proc_stdout.py", exit_callback=proc_exit_cb, stdout=stdout_pipe)
+            proc.spawn(file="./proc_stdout.py", exit_callback=proc_exit_cb, stdio=stdio)
         stdout_pipe.start_read(stdout_read_cb)
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
@@ -94,11 +98,14 @@ class ProcessTest(unittest2.TestCase):
             handle.close(handle_close_cb)
         loop = pyuv.Loop.default_loop()
         stdout_pipe = pyuv.Pipe(loop)
+        stdio = []
+        stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
+        stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
         proc = pyuv.Process(loop)
         if sys.platform == 'win32':
-            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_args_stdout.py", b"TEST"], exit_callback=proc_exit_cb, stdout=stdout_pipe)
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_args_stdout.py", b"TEST"], exit_callback=proc_exit_cb, stdio=stdio)
         else:
-            proc.spawn(file="./proc_args_stdout.py", args=[b"TEST"], exit_callback=proc_exit_cb, stdout=stdout_pipe)
+            proc.spawn(file="./proc_args_stdout.py", args=[b"TEST"], exit_callback=proc_exit_cb, stdio=stdio)
         stdout_pipe.start_read(stdout_read_cb)
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
@@ -120,18 +127,20 @@ class ProcessTest(unittest2.TestCase):
             handle.close(handle_close_cb)
         loop = pyuv.Loop.default_loop()
         stdout_pipe = pyuv.Pipe(loop)
+        stdio = []
+        stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
+        stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
         proc = pyuv.Process(loop)
         if sys.platform == 'win32':
-            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_env_stdout.py"], env={b"TEST": b"TEST"}, exit_callback=proc_exit_cb, stdout=stdout_pipe)
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_env_stdout.py"], env={b"TEST": b"TEST"}, exit_callback=proc_exit_cb, stdio=stdio)
         else:
-            proc.spawn(file="./proc_env_stdout.py", env={b"TEST": b"TEST"}, exit_callback=proc_exit_cb, stdout=stdout_pipe)
+            proc.spawn(file="./proc_env_stdout.py", env={b"TEST": b"TEST"}, exit_callback=proc_exit_cb, stdio=stdio)
         stdout_pipe.start_read(stdout_read_cb)
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
         self.assertEqual(self.received_output, b"TEST")
 
-    @platform_skip(["win32"])
     def test_process_stdin(self):
         self.exit_cb_called = 0
         self.close_cb_called = 0
@@ -154,11 +163,14 @@ class ProcessTest(unittest2.TestCase):
         loop = pyuv.Loop.default_loop()
         stdin_pipe = pyuv.Pipe(loop)
         stdout_pipe = pyuv.Pipe(loop)
+        stdio = []
+        stdio.append(pyuv.StdIO(stream=stdin_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_READABLE_PIPE))
+        stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
         proc = pyuv.Process(loop)
         if sys.platform == 'win32':
-            proc.spawn(file="cmd.exe", args=["/c", "proc_stdin_stdout.py"], exit_callback=proc_exit_cb, stdin=stdin_pipe, stdout=stdout_pipe)
+            proc.spawn(file="cmd.exe", args=["/c", "proc_stdin_stdout.py"], exit_callback=proc_exit_cb, stdio=stdio)
         else:
-            proc.spawn(file="./proc_stdin_stdout.py", exit_callback=proc_exit_cb, stdin=stdin_pipe, stdout=stdout_pipe)
+            proc.spawn(file="./proc_stdin_stdout.py", exit_callback=proc_exit_cb, stdio=stdio)
         stdout_pipe.start_read(stdout_read_cb)
         stdin_pipe.write(b"TEST"+common.linesep, stdin_write_cb)
         loop.run()
@@ -241,6 +253,24 @@ class ProcessTest(unittest2.TestCase):
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 1)
 
+    def test_process_detached(self):
+        self.exit_cb_called = 0
+        def proc_exit_cb(proc, exit_status, term_signal):
+            self.exit_cb_called += 1
+        loop = pyuv.Loop.default_loop()
+        proc = pyuv.Process(loop)
+        if sys.platform == 'win32':
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_basic.py"], exit_callback=proc_exit_cb, flags=pyuv.UV_PROCESS_DETACHED)
+        else:
+            proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb, flags=pyuv.UV_PROCESS_DETACHED)
+        proc.unref()
+        pid = proc.pid
+        loop.run()
+        self.assertEqual(self.exit_cb_called, 0)
+        proc.kill(0)
+        proc.kill(15)
+        self.assertNotEqual(pid, None)
+
 
 if __name__ == '__main__':
     unittest2.main(verbosity=2)
